<?php declare(strict_types=1);

namespace addianto\LionGame\Backend;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Slim\Factory\AppFactory;
use Slim\App as SlimApp;
use addianto\LionGame\Backend\Config\Config;
use addianto\LionGame\Backend\Service\SheetsService;
use count;
use json_encode;

class App
{
    const VALID_LENGTH = 29;
    private string $apiKey;
    private SlimApp $app;
    private SheetsService $sheets;
    private bool $debug;

    public function __construct(
        string $apiKey,
        SheetsService $sheets,
        bool $debug = false
    ) {
        $this->apiKey = $apiKey;
        $this->app = AppFactory::create();
        $this->sheets = $sheets;
        $this->debug = $debug;

        $this->setupMiddleware();
        $this->setupController();
    }

    public function get(): SlimApp
    {
        return $this->app;
    }

    private function setupMiddleware(): void
    {
        $this->app->addBodyParsingMiddleware();
        $this->app->addErrorMiddleware($this->debug, true, true);
    }

    private function setupController(): void
    {
        $this->app->post("/api/logs", function (Request $request, Response $response, $args): Response {
            if (!self::isAuthorized($request, $this->apiKey)) {
                self::buildErrorResponse($response, 403);
                return $response;
            }

            $data = $request->getParsedBody();

            if (!self::isValidData($data)) {
                self::buildErrorResponse($response, 400);
                return $response;
            }

            if ($this->sheets->appendValues(
                "'Play Session Analytics (Example)'!A:AC",
                array($data)
            )) {
                $response
                    ->withStatus(200)
                    ->getBody()->write(json_encode(array(
                        "status" => 200,
                        "data" => array(
                            "participant_name" => $data[0][4],
                            "created_at" => $data[0][26],
                        ),
                    )));

                return $response;
            } else {
                self::buildErrorResponse($response, 503);
                return $response;
            }
        });

        $this->app->post("/api/logs/batch", function (Request $request, Response $response, $args): Response {
            if (!self::isAuthorized($request, $this->apiKey)) {
                self::buildErrorResponse($response, 403);
                return $response;
            }

            $data = $request->getParsedBody();

            if (!self::isValidData($data)) {
                self::buildErrorResponse($response, 400);
                return $response;
            }

            if ($this->sheets->appendValues(
                "'Play Session Analytics (Example)'!A:AC",
                $data
            )) {
                $response
                    ->withStatus(200)
                    ->getBody()->write(json_encode(array(
                        "status" => 200,
                        "data" => array(
                            "participant_name" => $data[count($data)-1][4],
                            "created_at" => $data[count($data)-1][26],
                        ),
                    )));

                return $response;
            } else {
                self::buildErrorResponse($response, 503);
                return $response;
            }
        });
    }

    private static function buildErrorResponse(Response $response, int $code): void
    {
        $response->withStatus($code)->getBody()->write(json_encode(array(
            "status" => $code,
        )));
    }

    private static function isAuthorized(Request $request, string $apiKey): bool
    {
        if (!$request->hasHeader("Authorization")) {
            return false;
        }

        $headerValue = $request->getHeader("Authorization");

        return $headerValue[0] == $apiKey;
    }

    private static function isValidData(array $data): bool
    {
        if (is_array($data[0])) {
            foreach ($data as $entry) {
                if (count($entry) != self::VALID_LENGTH) {
                    return false;
                }
            }
            return true;
        }

        return count($data) == self::VALID_LENGTH;
    }
}
